import java.awt.Color;
import java.awt.BasicStroke;
import java.awt.Toolkit;
import java.awt.Dimension;
import java.awt.Container;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.Point2D;
import java.awt.geom.Line2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.PathIterator;
import java.awt.geom.AffineTransform;
import java.awt.event.KeyEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JColorChooser;
import javax.swing.KeyStroke;


public class GUI extends JFrame implements ActionListener {
	private GUI.LimitsDialog dLimits;
	private GUI.PanelDibujo pDib;
	private JButton btPlot, btReset;
	private JLabel lFunc;
	private JMenuItem miLimits, miZoomIn, miZoomOut, miUp, miDown, miLeft,
					  miRight, miAxisWidth, miAxisColor, miAxisPunt, miGridWidth,
					  miGridColor, miGridPunt, miGridToggle, miAAToggle,
					  miAnimToggle, miClose;
	private Color color1, color2;
	private float grosor1, grosor2;
	private BasicStroke stroke1, stroke2;
	private float xMin, yMin, xMax, yMax;
	private boolean punt1, punt2, showGrid, antiAlias, plot, anim;
	private Polinomio poly;


	public GUI() {
		super();
		setTitle("Representación Gráfica");
		Dimension screen_size = Toolkit.getDefaultToolkit().getScreenSize();
		setSize(screen_size.width/2, Math.round(screen_size.width/2.5f)); // 5/4
		setMinimumSize( new Dimension(screen_size.width/2, Math.round(screen_size.width/2.5f)) );
		setLocationRelativeTo(null);
		pDib = new PanelDibujo();
		JPanel pCtl = new JPanel();
		pCtl.setBackground(new Color(200, 225, 250));
		Container cont = getContentPane();
		cont.setLayout(new BorderLayout());
		cont.add(pDib, BorderLayout.CENTER);
		cont.add(pCtl, BorderLayout.SOUTH);

		// Menu
		JMenuBar mb = new JMenuBar();
		setJMenuBar(mb);
		JMenu showMenu = new JMenu("Ver");
		mb.add(showMenu);
		showMenu.setMnemonic('V');
		JMenu propMenu = new JMenu("Propiedades");
		propMenu.setMnemonic('P');
		mb.add(propMenu);
		JMenu exitMenu = new JMenu("Salir");
		exitMenu.setMnemonic('S');
		mb.add(exitMenu);
		showMenu.add(miLimits= new JMenuItem("Límites de Dibujado..."));
		showMenu.addSeparator();
		showMenu.add(miZoomIn= new JMenuItem("Acercar"));
		showMenu.add(miZoomOut= new JMenuItem("Alejar"));
		showMenu.addSeparator();
		showMenu.add(miUp = new JMenuItem("Arriba"));
		showMenu.add(miDown = new JMenuItem("Abajo"));
		showMenu.add(miLeft = new JMenuItem("Izquierda"));
		showMenu.add(miRight = new JMenuItem("Derecha"));
		propMenu.add(miAxisWidth = new JMenuItem("Grosor Ejes..."));
		propMenu.add(miAxisColor = new JMenuItem("Color Ejes..."));
		propMenu.add(miAxisPunt = new JMenuItem("Punteado Ejes..."));
		propMenu.addSeparator();
		propMenu.add(miGridWidth = new JMenuItem("Grosor Rejilla..."));
		propMenu.add(miGridColor = new JMenuItem("Color Rejilla..."));
		propMenu.add(miGridPunt = new JMenuItem("Punteado Rejilla..."));
		propMenu.addSeparator();
		propMenu.add(miGridToggle = new JMenuItem("Conmutar Rejilla"));
		propMenu.add(miAAToggle = new JMenuItem("Conmutar AntiAliasing"));
		propMenu.add(miAnimToggle = new JMenuItem("Conmutar Animación"));
		exitMenu.add(miClose = new JMenuItem("Cerrar"));
		miLimits.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.CTRL_MASK));
		miZoomIn.setAccelerator(KeyStroke.getKeyStroke('+'));
		miZoomOut.setAccelerator(KeyStroke.getKeyStroke('-'));
		miUp.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_UP, 0));
		miDown.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, 0));
		miLeft.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, 0));
		miRight.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, 0));
		miGridToggle.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G, ActionEvent.CTRL_MASK));
		miAAToggle.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK));
		miAnimToggle.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_M, ActionEvent.CTRL_MASK));
		miClose.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK));

		// Panel de Control
		pCtl.add( btPlot = new JButton("Plot") );
		pCtl.add( lFunc = new JLabel() );
		pCtl.add( btReset = new JButton("Reset") );
		btPlot.setPreferredSize(new Dimension(75,22));
		lFunc.setPreferredSize(new Dimension(160,22));
		btReset.setPreferredSize(new Dimension(75,22));
		lFunc.setHorizontalAlignment(JLabel.CENTER);



		// Registra a 'GUI' (this) como oyente de cada boton y menú
		btPlot.addActionListener(this);
		btReset.addActionListener(this);
		miLimits.addActionListener(this);
		miZoomIn.addActionListener(this);
		miZoomOut.addActionListener(this);
		miUp.addActionListener(this);
		miDown.addActionListener(this);
		miLeft.addActionListener(this);
		miRight.addActionListener(this);
		miAxisWidth.addActionListener(this);
		miAxisColor.addActionListener(this);
		miAxisPunt.addActionListener(this);
		miGridWidth.addActionListener(this);
		miGridColor.addActionListener(this);
		miGridPunt.addActionListener(this);
		miGridToggle.addActionListener(this);
		miAAToggle.addActionListener(this);
		miAnimToggle.addActionListener(this);
		miClose.addActionListener(this);

		// Inicializa los parámetros iniciales y el 'Custom Dialog' de Límites
		initValues();
		dLimits = new LimitsDialog(this, true);
	}


	public void actionPerformed(ActionEvent e) {
		String ac = e.getActionCommand();

		if ("Plot".equals(ac)) {
			plot = true;
			poly = new Polinomio( randomArray( (int)(Math.random() * 5) + 1 ) );
			lFunc.setText(poly.toString());
			pDib.repaint();
		} else if ("Reset".equals(ac)) {
			initValues();
			lFunc.setText("");
			pDib.repaint();
		} else if ("Acercar".equals(ac))
			zoom(true);
		else if ("Alejar".equals(ac))
			zoom(false);
		else if ("Arriba".equals(ac))
			mover('N');
		else if ("Abajo".equals(ac))
			mover('S');
		else if ("Izquierda".equals(ac))
			mover('E');
		else if ("Derecha".equals(ac))
			mover('O');
		else if ("Límites de Dibujado...".equals(ac))
			showDialogLimits();
		else if ("Grosor Ejes...".equals(ac))
			showDialogGrosor(true);
		else if ("Color Ejes...".equals(ac))
			showDialogColor(true);
		else if ("Punteado Ejes...".equals(ac))
			showDialogPunteado(true);
		else if ("Grosor Rejilla...".equals(ac))
			showDialogGrosor(false);
		else if ("Color Rejilla...".equals(ac))
			showDialogColor(false);
		else if ("Punteado Rejilla...".equals(ac))
			showDialogPunteado(false);
		else if ("Conmutar Rejilla".equals(ac)) {
			showGrid = (showGrid) ? false : true;
			pDib.repaint();
		} else if ("Conmutar AntiAliasing".equals(ac)) {
			antiAlias = (antiAlias) ? false : true;
			pDib.repaint();
		} else if ("Conmutar Animación".equals(ac)) {
			anim = (anim) ? false : true;
			pDib.repaint();
		} else if ("Cerrar".equals(ac))
			System.exit(0);
	}


	// Muestra el Diálgo para especificar los Límites de dibujado
	public void showDialogLimits() {
		dLimits.setVisible(false);
		dLimits.refreshTextFields();
		dLimits.setVisible(true);
	}


	// Realiza Zoom hacia dentro o hacia fuera
	public void zoom(boolean in) {
		if (in) {
			xMin /= 2.0f;
			xMax /= 2.0f;
			yMin /= 2.0f;
			yMax /= 2.0f;
		} else {
			xMin *= 2.0f;
			xMax *= 2.0f;
			yMin *= 2.0f;
			yMax *= 2.0f;
		}
		pDib.repaint();
	}


	// Desplaza el dibujo en las 4 posibles direcciones
	public void mover(char dir) {
		float pasoX = (xMax - xMin) / 4.0f;
		float pasoY = (yMax - yMin) / 4.0f;
		switch (dir) {
			case 'N':
				yMin += pasoY;
				yMax += pasoY;
				break;
			case 'S':
				yMin -= pasoY;
				yMax -= pasoY;
				break;
			case 'O':
				xMin += pasoX;
				xMax += pasoX;
				break;
			case 'E':
				xMin -= pasoX;
				xMax -= pasoX;
				break;
		}
		pDib.repaint();
	}


	// Muestra el Diálogo para elegir el Grosor de las líneas
	public void showDialogGrosor(boolean eje) {
		float gr = (eje) ? grosor1 : grosor2;
		String strError = new String();
		String strIn = Float.toString(gr);
		String strOut = JOptionPane.showInputDialog(
							this,
							"Introduzca el grosor de la línea:",
							strIn
						);
		if ( (strOut != null) && (strIn != strOut) ) {
			try {
				gr = Float.parseFloat(strOut);
				if (eje) {
					grosor1 = gr;
					if (punt1)
						stroke1 = new BasicStroke(gr, BasicStroke.CAP_BUTT,
												BasicStroke.JOIN_ROUND, 1.0f,
												new float[] {10, 5, 5, 5}, 2);
					else
						stroke1 = new BasicStroke(gr);
				} else {
					grosor2 = gr;
					if (punt2)
						stroke2 = new BasicStroke(gr, BasicStroke.CAP_BUTT,
												BasicStroke.JOIN_ROUND, 1.0f,
												new float[] {10, 5, 5, 5}, 2);
					else
						stroke2 = new BasicStroke(gr);
				}
				pDib.repaint();
			} catch (NumberFormatException ex) {
				if (ex.getMessage() == "empty String")
					strError = "El campo está vacío";
				else
					strError = "Error de Formato Numérico";
				JOptionPane.showMessageDialog(
					this,
					strError,
					"RunTime Error",
					JOptionPane.ERROR_MESSAGE
				);
			}
		}
	}


	// Muestra el Diálogo para preguntar el Color de las líneas
	public void showDialogColor(boolean eje) {
		Color colrIn = (eje) ? color1 : color2;
		Color colrOut = JColorChooser.showDialog(this, "Color", colrIn);
		if ( (colrOut != null) && (colrIn != colrOut) ) {
			if (eje)
				color1 = colrOut;
			else
				color2 = colrOut;
			pDib.repaint();
		}
	}


	// Muestra el Diálogo para preguntar si las líneas son Punteadas
	public void showDialogPunteado(boolean eje) {
		boolean punt = false;
		float gr = (eje) ? grosor1 : grosor2;
		BasicStroke strk = new BasicStroke();
		int n = JOptionPane.showConfirmDialog(
					this,
					"¿Desea que la línea sea punteada?",
					"Punteado",
					JOptionPane.YES_NO_CANCEL_OPTION
				);
		switch(n) {
			case 0:
				strk = new BasicStroke( gr, BasicStroke.CAP_BUTT,
										BasicStroke.JOIN_ROUND, 1.0f,
										new float[] {10, 5, 5, 5}, 2 );
				punt = true;
				break;
			case 1:
				strk = new BasicStroke(gr);
				punt = false;
				break;
		}
		if (n != 2) {
			if (eje) {
				stroke1 = strk;
				punt1 = punt;
			} else {
				stroke2 = strk;
				punt2 = punt;
			}
			pDib.repaint();
		}
	}


	// Reset de todos los valores
	public void initValues() {
		xMin = -3.2f;
		xMax = 3.2f;
		yMin = -2.2f;
		yMax = 2.2f;
		color1 = Color.RED;
		color2 = Color.LIGHT_GRAY;
		grosor1 = 2.0f;
		grosor2 = 1.0f;
		punt1 = false;
		punt2 = true;
		stroke1 = new BasicStroke(grosor1);
		stroke2 = new BasicStroke(grosor2, BasicStroke.CAP_BUTT,
								  BasicStroke.JOIN_ROUND, 1.0f,
								  new float[] {10, 5, 5, 5}, 2);
		showGrid = true;
		antiAlias = true;
		plot = false;
		anim = true;
	}


	// Genera un array de 'n' enteros, siendo estos aleatorios desde -4 a +4
	public int[] randomArray(int n) {
		int[] x = new int[n];
		for (int i = 0; i < n; i++)
			x[i] = (int) ( (Math.random() * 8) - 4 );
		return x;
	}


	// Método Main
	public static void main(String[] args) {
		GUI f = new GUI();
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		f.setVisible(true);
	}





	class PanelDibujo extends JPanel {
		private Graphics2D g2;
		private AffineTransform at;
		private GeneralPath pathGrid;
		private GeneralPath pathFunc;
		private ThreadGrid tg;
		private ThreadFunc tf;
		private ThreadAnim ta;
		private float paso;


		public PanelDibujo() {
			super();
			setBackground(new Color(235, 255, 215));
		}


		public /*synchronized*/ void paintComponent(Graphics g) {
			super.paintComponent(g);
			g2 = (Graphics2D) g;
			if (antiAlias) {
				g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
									RenderingHints.VALUE_ANTIALIAS_ON);
				g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
									RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
			}
			tg = new ThreadGrid();
			tf = new ThreadFunc();
			paso = (GUI.this.xMax - GUI.this.xMin) / (getWidth()/2.0f);


			// Aplica la Transformación a los límites establecidos
			at = new AffineTransform();
			at.translate( getWidth() / 2, getHeight() / 2 );
			at.scale( getWidth() / (GUI.this.xMax - GUI.this.xMin),
					  -getHeight() / (GUI.this.yMax - GUI.this.yMin) );
			at.translate( (GUI.this.xMin + GUI.this.xMax) / -2,
						  (GUI.this.yMin + GUI.this.yMax) / -2);

			// Computa y Dibuja la Rejilla con un GeneralPath en un THREAD
			if ( (showGrid) && ( (GUI.this.xMax-GUI.this.xMin < 100.0f) ||
								 (GUI.this.yMax-GUI.this.yMin < 100.0f) ) ) {
				pathGrid = new GeneralPath();
				tg.start();
			}

			// Computa la función (Polinomio) con un GeneralPath en un THREAD
			if (GUI.this.plot) {
				pathFunc = new GeneralPath();
				tf.start();
			}

			// Espera a que acabe el Thread de la Rejilla
			while ( tg.isAlive() )
				;

			// Dibuja los Ejes con los límites de Dibujado
			drawAxis();

			// Dibuja la Función
			if (GUI.this.plot) {
				g2.setPaint(Color.BLACK);
				g2.setStroke(new BasicStroke(GUI.this.grosor1+1,
											BasicStroke.CAP_ROUND,
											BasicStroke.JOIN_ROUND));
				// Espera a que acabe el Thread que calcula la Función
				while ( tf.isAlive() )
					;
				if (GUI.this.anim) {
					ta = new ThreadAnim();
					ta.start();
					while ( ta.isAlive() )
						;
				}
				else
					g2.draw(pathFunc);
			}
		}


		// Método que dibuja los Ejes, así como sus valores límite
		private void drawAxis() {
			Line2D.Float line = new Line2D.Float();
			Point2D.Float p = new Point2D.Float(0.0f, 0.0f);
			boolean xEje, yEje;
			String str = new String();
			float c = (float) Math.ceil(GUI.this.grosor1);
			int w, h;

			FontMetrics metrics = g2.getFontMetrics();
			at.transform(p, p);
			xEje = yEje = false;

			// Dibuja el Eje X y sus valores límite (si aplicable)
			if ( (GUI.this.yMin <= 0.0f) && (GUI.this.yMax >= 0.0f) ) {
				line.setLine(0.0f, p.y, getWidth(), p.y);
				g2.setPaint(GUI.this.color1);
				g2.setStroke(GUI.this.stroke1);
				g2.draw(line);

				// Dibuja los valores límite del eje X
				xEje = true;
				g2.setColor(Color.BLUE);
				if (p.y < 20) {
					str = Float.toString( round4dec(GUI.this.xMin) );
					w = metrics.stringWidth(str);
					h = metrics.getHeight() - metrics.getDescent();
					g2.drawString( str, 0.0f, p.y+h+c );
					str = Float.toString( round4dec(GUI.this.xMax) );
					w = metrics.stringWidth(str);
					h = metrics.getHeight() - metrics.getDescent();
					g2.drawString( str, getWidth()-w, p.y+h+c );
				} else {
					str = Float.toString( round4dec(GUI.this.xMin) );
					w = metrics.stringWidth(str);
					h = metrics.getHeight() - metrics.getAscent();
					g2.drawString( str, 0.0f, p.y-h/2-c );
					str = Float.toString( round4dec(GUI.this.xMax) );
					w = metrics.stringWidth(str);
					h = metrics.getHeight() - metrics.getAscent();
					g2.drawString( str, getWidth()-w, p.y-h/2-c );
				}
			}

			// Dibuja el Eje Y y sus valores límite (si aplicable)
			if ( (GUI.this.xMin <= 0.0f) && (GUI.this.xMax >= 0.0f) ) {
				line.setLine(p.x, 0.0f, p.x, getHeight());
				g2.setPaint(GUI.this.color1);
				g2.setStroke(GUI.this.stroke1);
				g2.draw(line);

				// Dibuja los valores límite del eje Y
				yEje = true;
				g2.setColor(Color.BLUE);
				if (p.x < 20) {
					str = Float.toString( round4dec(GUI.this.yMin) );
					w = metrics.stringWidth(str);
					h = metrics.getHeight() - metrics.getAscent();
					g2.drawString( str, p.x+w/12+c, getHeight()-h/2 );
					str = Float.toString( round4dec(GUI.this.yMax) );
					w = metrics.stringWidth(str);
					h = metrics.getHeight() - metrics.getDescent();
					g2.drawString( str, p.x+w/12+c, h );
				} else {
					str = Float.toString( round4dec(GUI.this.yMin) );
					w = metrics.stringWidth(str);
					h = metrics.getHeight() - metrics.getAscent();
					g2.drawString( str, p.x-w-c, getHeight()-h/2 );
					str = Float.toString( round4dec(GUI.this.yMax) );
					w = metrics.stringWidth(str);
					h = metrics.getHeight() - metrics.getDescent();
					g2.drawString( str, p.x-w-c, h );
				}

			}

			// Dibuja los valores límites que no fueron dibujados con los Ejes
			g2.setColor(Color.BLUE);
			if (! xEje) {
				str = Float.toString( round4dec(GUI.this.xMin) );
				w = metrics.stringWidth(str);
				h = metrics.getHeight() - metrics.getDescent();
				g2.drawString( str, 0, getHeight()/2+h/2 );
				str = Float.toString( round4dec(GUI.this.xMax) );
				w = metrics.stringWidth(str);
				h = metrics.getHeight() - metrics.getDescent();
				g2.drawString( str, getWidth()-w, getHeight()/2+h/2 );
			}
			if (! yEje) {
				str = Float.toString( round4dec(GUI.this.yMin) );
				w = metrics.stringWidth(str);
				h = metrics.getHeight() - metrics.getAscent();
				g2.drawString( str, getWidth()/2-w/2, getHeight()-h/2 );
				str = Float.toString( round4dec(GUI.this.yMax) );
				w = metrics.stringWidth(str);
				h = metrics.getHeight() - metrics.getDescent();
				g2.drawString( str, getWidth()/2-w/2, h );
			}
		}


		private float round4dec(float f) {
			f *= 10000.0f;
			int r = Math.round(f);
			return (float) r / 10000.0f;
		}



		// Clase para calcular y dibujar la Rejilla en un Thread
		private class ThreadGrid extends Thread {
			public void run() {
				// Computa las líneas de la Rejilla (unidades) del eje X
				for (float n = (float) Math.ceil(GUI.this.yMin); n < GUI.this.yMax; n += 1.0f) {
					pathGrid.moveTo(GUI.this.xMin, n);
					pathGrid.lineTo(GUI.this.xMax, n);
				}
				// Computa las líneas de la Rejilla (unidades) del eje Y
				for (float n = (float) Math.ceil(GUI.this.xMin); n < GUI.this.xMax; n += 1.0f) {
					pathGrid.moveTo(n, GUI.this.yMin);
					pathGrid.lineTo(n, GUI.this.yMax);
				}
				pathGrid.transform(at);
				g2.setPaint(GUI.this.color2);
				g2.setStroke(GUI.this.stroke2);
				g2.draw(pathGrid);
			}
		}



		private class ThreadFunc extends Thread {
			public void run() {
				float y = 0.0f;
				float correctY = (GUI.this.yMax - GUI.this.yMin) / 16.0f;

				pathFunc.moveTo(GUI.this.xMin, GUI.this.poly.evalue(GUI.this.xMin));
				for (float x = GUI.this.xMin; x < GUI.this.xMax ; x += paso ) {
					y = GUI.this.poly.evalue(x);
					if ( y > (GUI.this.yMax + correctY) )
						y = GUI.this.yMax + correctY;
					else if ( y < (GUI.this.yMin - correctY) )
						y = GUI.this.yMin - correctY;
					pathFunc.lineTo(x, y);
				}
				pathFunc.transform(at);
			}
		}



		private class ThreadAnim extends Thread {
			public void run() {
				Line2D.Float line = new Line2D.Float();
				PathIterator pi = pathFunc.getPathIterator(null);
				float[] coord = new float[6];
				float[] coordAnt = new float[2];

				try {
					for (int i = 0; !pi.isDone(); i++) {
						pi.currentSegment(coord);
						if (i == 0) {
							coordAnt[0] = coord[0];
							coordAnt[1] = coord[1];
						} else {
							line.setLine(coordAnt[0], coordAnt[1],
										 coord[0], coord[1]);
							g2.draw(line);
							coordAnt[0] = coord[0];
							coordAnt[1] = coord[1];
						}
						if ( ((i % 5) == 0) && (i != 0) ) {
							sleep(10);
						}
						pi.next();
					}
				} catch (InterruptedException e) {
					System.out.println("Interrupted Exception");
				}
			}
		}
	}




	public class LimitsDialog extends JDialog implements ActionListener {
		private JTextField xMin, xMax, yMin, yMax;
		private JButton btAcept, btCancl;


		public LimitsDialog(JFrame owner, boolean modal) {
			super( owner, "Límites", modal );
			setSize(250, 200);
			setResizable(false);
			setLocationRelativeTo(owner);
			JPanel pInp = new JPanel();
			JPanel pCtl = new JPanel();
			pCtl.setBackground(new Color(200, 225, 250));
			Container cont = getContentPane();
			cont.setLayout(new BorderLayout(10, 10));
			cont.add(pInp, BorderLayout.CENTER);
			cont.add(pCtl, BorderLayout.SOUTH);
			pInp.setLayout( new GridLayout(4, 2, 20, 20) ) ;
			pInp.add( new JLabel("  X Mínimo:") );
			pInp.add( xMin = new JTextField(10) );
			pInp.add( new JLabel("  X Máximo:") );
			pInp.add( xMax = new JTextField(10) );
			pInp.add( new JLabel("  Y Mínimo:") );
			pInp.add( yMin = new JTextField(10) );
			pInp.add( new JLabel("  Y Máximo:") );
			pInp.add( yMax = new JTextField(10) );
			pCtl.add( btAcept = new JButton("Aceptar") );
			pCtl.add( btCancl = new JButton("Cancelar") );
			btAcept.addActionListener(this);
			btCancl.addActionListener(this);
		}


		public void refreshTextFields() {
			xMin.setText( Float.toString(GUI.this.xMin) );
			xMax.setText( Float.toString(GUI.this.xMax) );
			yMin.setText( Float.toString(GUI.this.yMin) );
			yMax.setText( Float.toString(GUI.this.yMax) );
		}


		public void actionPerformed(ActionEvent ae) {
			String ac = ae.getActionCommand();
			String strError = new String();
			float tmp;

			if ("Aceptar".equals(ac)) {
				try {
					strError = "X Mínimo";
					GUI.this.xMin = Float.parseFloat( xMin.getText().trim() );
					strError = "X Máximo";
					GUI.this.xMax = Float.parseFloat( xMax.getText().trim() );
					strError = "Y Mínimo";
					GUI.this.yMin = Float.parseFloat( yMin.getText().trim() );
					strError = "Y Máximo";
					GUI.this.yMax = Float.parseFloat( yMax.getText().trim() );
					// if (xMin > xMax) => Swap
					if (GUI.this.xMin > GUI.this.xMax) {
						tmp = GUI.this.xMin;
						GUI.this.xMin = GUI.this.xMax;
						GUI.this.xMax = tmp;
					}
					// if (yMin > yMax) => Swap
					if (GUI.this.yMin > GUI.this.yMax) {
						tmp = GUI.this.yMin;
						GUI.this.yMin = GUI.this.yMax;
						GUI.this.yMax = tmp;
					}
					GUI.this.pDib.repaint();
					this.setVisible(false);
				} catch (NumberFormatException ex) {
					if (ex.getMessage() == "empty String")
						strError = "El campo '" + strError + "' está vacío";
					else
						strError = "Error de Formato Numérico en el campo '"
								   + strError + "'";

					JOptionPane.showMessageDialog(
						this,
						strError,
						"RunTime Error",
						JOptionPane.ERROR_MESSAGE
					);
				}
			}
			else if ("Cancelar".equals(ac))
				this.setVisible(false);
		}
	}
}